﻿using System;
using System.Threading;
using System.Collections;

using Holo.Managers;
using Holo.Virtual.Rooms;

namespace Holo.Virtual.Users
{
    /// <summary>
    /// Represents a virtual user in a virtual room.
    /// </summary>
    public class virtualRoomUser
    {
        /// <summary>
        /// The database ID of the user.
        /// </summary>
        internal int userID;
        /// <summary>
        /// The ID of the virtual room where the user is in.
        /// </summary>
        internal int roomID;
        /// <summary>
        /// The ID of the user in the virtual room.
        /// </summary>
        internal int roomUID;
        internal string swimOutfit;
        /// <summary>
        /// The X position of the user in the virtual room.
        /// </summary>
        internal int X;
        /// <summary>
        /// The Y position of the user in the virtual room.
        /// </summary>
        internal int Y;
        /// <summary>
        /// The height of the user in the virtual room.
        /// </summary>
        internal double H;
        /// <summary>
        /// The rotation of the user's head in the virtual room.
        /// </summary>
        internal byte Z1;
        /// <summary>
        /// The rotation of the user's body in the virtual room.
        /// </summary>
        internal byte Z2;
        /// <summary>
        /// The X position where the user wants to walk to. If not walking, -1.
        /// </summary>
        internal int goalX;
        /// <summary>
        /// The Y position where the user wants to walk to. If not walking, -1.
        /// </summary>
        internal int goalY;
        /// <summary>
        /// Specifies if the user can walk.
        /// </summary>
        internal bool walkLock;
        /// <summary>
        /// Specifies if the user is on his way of leaving the room after clicking the door.
        /// </summary>
        internal bool walkDoor;
        /// <summary>
        /// Guestroom only. Specifies if the user has voted on the guestroom already.
        /// </summary>
        internal bool hasVoted;
        /// <summary>
        /// Swimming pools only. Specifies the swimming pool outfit.
        /// </summary>
        internal string SwimOutfit;
        /// <summary>
        /// Gamelobbies only. Specifies the amount of gamepoints that this user has for the game in the current gamelobby.
        /// </summary>
        internal int gamePoints;
        /// <summary>
        /// The parent virtual user of this room user.
        /// </summary>
        internal virtualUser User;
        /// <summary>
        /// The status manager of this room user.
        /// </summary>
        internal virtualRoomUserStatusManager statusManager;
        /// <summary>
        /// Indicates if the room user is currently typing a chat message.
        /// </summary>
        internal bool isTyping;

        /// Indicates if the room user is able to use the special teleport cast, which allows him to teleport around the room.
        /// </summary>
        internal bool SPECIAL_TELEPORTABLE;




        // WOO!!!!



        /// <summary>
        /// Turn the pettransform on or off.
        /// </summary>
        internal bool petTransform_cat;
        /// <summary>
        /// Turn the pettransform on or off.
        /// </summary>
        internal bool petTransform_dog;
        /// <summary>
        /// Turn the pettransform on or off.
        /// </summary>
        internal bool petTransform_croc;
        /// <summary>
        /// Initializes a virtual room user.
        /// </summary>
        /// <param name="userID">The database ID of the virtual user of this room user object.</param>
        /// <param name="roomID">The database ID of the room where the room user is in.</param>
        /// <param name="roomUID">The ID that identifies the room user in the virtual room.</param>
        /// <param name="User">The parent vittual user of this room user.</param>
        /// <param name="statusManager">The status manager of this room user.</param>
        internal virtualRoomUser(int userID, int roomID, int roomUID, virtualUser User, virtualRoomUserStatusManager statusManager)
        {
            this.userID = userID;
            this.roomID = roomID;
            this.roomUID = roomUID;
            this.User = User;
            this.statusManager = statusManager;
        }
        /// <summary>
        /// The virtualRoom object that represents the room where this room user is in.
        /// </summary>
        internal virtualRoom Room
        {
            get
            {
                return roomManager.getRoom(roomID);
            }
        }
        /// <summary>
        /// Refreshes the status of this room user in the virtual room.
        /// </summary>
        internal void Refresh()
        {
            Room.Refresh(this);
        }
        /// <summary>
        /// Returns the details string for this room user, containing username etc.
        /// </summary>
        internal string detailsString
        {
            get
            {
                if (User._isInvisible == false)
                {
                    if (User.imitateEnabled)
                    {
                        try
                        {
                            if (userManager.containsUser(User.imitateID))
                            {   // see if the user is online for their info
                                virtualUser Target = userManager.getUser(User.imitateID);
                                string i = "i:" + roomUID + Convert.ToChar(13) + "a:" + userID + Convert.ToChar(13) + "n:" + Target._Username + Convert.ToChar(13) + "f:" + Target._Figure + Convert.ToChar(13) + "l:" + X + " " + Y + " " + H + Convert.ToChar(13);
                                if (Target._Mission != "") { i += "c:" + Target._Mission + Convert.ToChar(13); }
                                if (Target._Badge != "") { i += "b:" + Target._Badge + Convert.ToChar(13); }
                                if (Target.roomUser.swimOutfit != "") { i += "p:" + Target.roomUser.swimOutfit + Convert.ToChar(13); }
                                if (Target._groupID > 0) { i += "g:" + Target._groupID + Convert.ToChar(13) + "t:" + (int)Target._groupMemberRank + Convert.ToChar(13); }
                                return i;
                            }
                            else
                            {   // if not, use their database info
                                string[] Target = DB.runReadRow("SELECT name,figure,mission,figure_swim FROM users WHERE id = '" + User.imitateID + "'");
                                string TargetBadge = "";
                                if (DB.checkExists("SELECT badgeid FROM users_badges WHERE userid = '" + User.imitateID + "' AND iscurrent = '1'"))
                                    TargetBadge = DB.runRead("SELECT badgeid FROM users_badges WHERE userid = '" + User.imitateID + "' AND iscurrent = '1'");
                                string i = "i:" + roomUID + Convert.ToChar(13) + "a:" + userID + Convert.ToChar(13) + "n:" + Target[0] + Convert.ToChar(13) + "f:" + Target[1] + Convert.ToChar(13) + "l:" + X + " " + Y + " " + H + Convert.ToChar(13);
                                if (Target[2] != "") { i += "c:" + Target[2] + Convert.ToChar(13); }
                                if (TargetBadge != "") { i += "b:" + TargetBadge + Convert.ToChar(13); }
                                if (Target[3] != "") { i += "p:" + Target[3] + Convert.ToChar(13); }
                                return i;
                            }
                        }
                        catch { return ""; }
                    }

                    string petTransformAdd = "";
                    string petTransformFigure = User._Figure;

                    if (User._isPet == true)
                    {
                        petTransformAdd = "7ui1";
                        petTransformFigure = User._petType + " 000 " + User._petColour;
                    }


                    string userName = DB.runRead("SELECT name FROM users WHERE id = '" + userID + "'");
                    string userMotto = DB.runRead("SELECT mission FROM users WHERE id = '" + userID + "'");
                    string userSex = DB.runRead("SELECT sex FROM users WHERE id = '" + userID + "'");
                    string s = "i:" + roomUID + Convert.ToChar(13) + "a:" + userID + Convert.ToChar(13) + "n:" + petTransformAdd + userName + Convert.ToChar(13) + "f:" + petTransformFigure + Convert.ToChar(13) + "l:" + X + " " + Y + " " + H + Convert.ToChar(13) + "c:" + userMotto + Convert.ToChar(13) + "s:" + userSex + Convert.ToChar(13);
                    //string s = "i:" + roomUID + Convert.ToChar(13) + "a:" + userID + Convert.ToChar(13) + "n:" + petTransformAdd + User._Username + Convert.ToChar(13) + "f:" + petTransformFigure + Convert.ToChar(13) + "l:" + X + " " + Y + " " + H + Convert.ToChar(13);
                    if (User._Mission != "") { s += "c:" + User._Mission + Convert.ToChar(13); }

                    s += "b:";
                    if (User._Badges.Count > 0)
                    {
                        for (int i = 0; i < User._Badges.Count; i++)
                        {
                            if (User._badgeSlotIDs[i] > 0)
                            {
                                s += User._badgeSlotIDs[i] + ":";
                                s += User._Badges[i];
                                if (i < User._Badges.Count)
                                    s += ",";
                            }
                        }
                    }
                    s += Convert.ToChar(13);

                    //if (User._Badge != "") { s += "b:" + User._Badge + Convert.ToChar(13); }
                    if (swimOutfit != "") { s += "p:" + swimOutfit + Convert.ToChar(13); }
                    if (User._groupID > 0) { s += "g:" + User._groupID + Convert.ToChar(13) + "t:" + (int)User._groupMemberRank + Convert.ToChar(13); }
                    return s;

                }
                else
                {
                    // ' + "l:" + X + " " + Y + " " + H + Convert.ToChar(13)'
                    return "i:" + roomUID + Convert.ToChar(13) + "a:" + userID + Convert.ToChar(13) + "n: " + Convert.ToChar(13) + "f:hd" + Convert.ToChar(13) + "[bot]" + Convert.ToChar(13) + "l:" + X + " " + Y + " 100" + Convert.ToChar(13);
                }
            }
        }

        /// <summary>
        /// Returns the action string for this room user, containing position, movements, statuses etc.
        /// </summary>
        internal string statusString
        {
            get
            {
               return roomUID + " " + X + "," + Y + "," + H.ToString().Replace(",",".") + "," + Z1 + "," + Z2 + "/" + statusManager.ToString();
            }
        }
    }
}
